from PyTASTileSystem import *
import math
import sys
import getopt
import copy


color_aqua = 'rgb(0,255,255)'
color_yellow = 'rgb(255,255,0)'
color_pink = 'rgb(255,0,255)'
color_grey = 'rgb(128,128,128)'
color_purple = 'rgb(127,0,255)'
color_red = 'rgb(255,0,0)'

color_map = {'00B': 'Red', '01B': 'Green', '10B': 'Black', '11B': 'White',
             '00I': color_pink, '01I': 'Blue', '10I': color_yellow, '11I': color_aqua}

def DisplayHelp():
    print("\nThis script takes as input a pattern and a number of times to repeat it and possibly the names")
    print("of output files, and outputs files for an aTAM system that creates a grid of that pattern.")
    print("\nUsage:")
    print("-h, --help\t\t\t\t\tDisplays this help message.")
    print("-p <pattern>, --pattern=<pattern>\t\t[Optional] String representing the pattern to be repeatedly")
    print("\t\t\t\t\t\t output. Default is '01101011'")
    print("-r <repeitions>, --reps=<repetitions>\t\t[Optional] The number of times to repeat the pattern.")
    print("\t\t\t\t\t\t Default value is 10.")
    print("-o <out file name>, --out=<out file name>\t[Optional] Base name of output files. If not specified,")
    print("\t\t\t\t\t\t`pattern{pattern}_x{num_repeitions}.tdp' will be used.")
    print("-d <dir>, --dir=<dir>\t\t\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\t\tthe current directory will be used.")
    print("-s, --signals\t\t\t\t\t[Optional] If this option is specified, the growth signal")
    print("\t\t\t\t\t\ttiles will be colored red.")
    print("")
    exit()


def ColorMapper(row_bit, col_bit, row_mark, col_mark):    
    if ('B' in row_mark) or ('B' in col_mark):
        mark = 'B'
    else:
        mark = 'I'
    color = color_map[f'{row_bit}{col_bit}{mark}']
    return color

def GetInitialRowTiles(pattern, color_mapper, chunk_len, diagonal_signal):
    tile_set = dict()
    tile_list = []

    row = None
    for i in range(0, len(pattern), chunk_len):

        curr = pattern[i:i+chunk_len]
        col = int(curr[0])
        if row is None:
            row = col
        col_mark = curr[1]
        row_mark = 'B'

        tile_type_name = f'Init{i}'
        tile_type_label = f'{col}{col_mark}'
        t = PyTASTileType(name=tile_type_name, label=tile_type_label)

        if i == 0:
            t.sides[S].label = f'{col}{col_mark}'
            t.sides[S].strength = 1
        elif i == chunk_len:
            t.sides[S].label = f'{col}{col_mark}{diagonal_signal}'
            t.sides[S].strength = 2
        else:
            t.sides[S].label = f'{col}{col_mark}'
            t.sides[S].strength = 1
        if i > 0:
            t.sides[W].strength = 2
            t.sides[W].label = f'init{i-chunk_len}'
        t.sides[E].label = f'init{i}'
        t.sides[E].strength = 2
        t.color = color_mapper(row, col, row_mark, col_mark)
        tile_set[tile_type_name] = t
        tile_list.append(t)
    return tile_set, tile_list

def GetInteriorTiles(color_mapper, diagonal_signal):
    tile_set = dict()

    for col in [0,1]:
        for col_mark in ['B', 'I']:

            # Make the tile types that get the star from the north and pass it east
            tile_type_name = f'{col}{col_mark}{col}{col_mark}{diagonal_signal}'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label)
            t.sides[N].label = f'{col}{col_mark}{diagonal_signal}'
            t.sides[N].strength = 2
            t.sides[E].label = f'{col}{col_mark}{diagonal_signal}'
            t.sides[E].strength = 1
            t.sides[W].label = f'<{col}{col_mark}'
            t.sides[W].strength = 1
            t.sides[S].label = f'{col}{col_mark}'
            t.sides[S].strength = 1
            t.color = color_mapper(col, col, col_mark, col_mark)
            tile_set[tile_type_name] = t

            for row in [0,1]:
                for row_mark in ['B', 'I']:
                    tag = f'{row}{row_mark}{col}{col_mark}'
                    
                    # Make tiles for the left side of the diagonal
                    tile_type_name = f'<{tag}'
                    tile_type_label = tile_type_name
                    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
                    t.sides[N].label = f'{col}{col_mark}'
                    t.sides[N].strength = 1
                    t.sides[E].label = f'<{row}{row_mark}'
                    t.sides[E].strength = 1
                    t.sides[W].label = f'<{row}{row_mark}'
                    t.sides[W].strength = 1
                    t.sides[S].label = f'{col}{col_mark}'
                    t.sides[S].strength = 1
                    t.color = color_mapper(row, col, row_mark, col_mark)
                    tile_set[tile_type_name] = t

                    # Make tiles for the right side of the diagonal
                    tile_type_name = f'{tag}>'
                    tile_type_label = tile_type_name
                    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
                    t.sides[N].label = f'{col}{col_mark}'
                    t.sides[N].strength = 1
                    t.sides[E].label = f'{row}{row_mark}>'
                    t.sides[E].strength = 1
                    t.sides[W].label = f'{row}{row_mark}>'
                    t.sides[W].strength = 1
                    t.sides[S].label = f'{col}{col_mark}'
                    t.sides[S].strength = 1
                    t.color = color_mapper(row, col, row_mark, col_mark)
                    tile_set[tile_type_name] = t

                    # Make the tile types that get the star from the west and pass it south
                    tile_type_name = f'{tag}>{diagonal_signal}'
                    tile_type_label = tile_type_name
                    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
                    t.sides[N].label = f'{col}{col_mark}'
                    t.sides[N].strength = 1
                    t.sides[E].label = f'{row}{row_mark}>'
                    t.sides[E].strength = 1
                    t.sides[W].label = f'{row}{row_mark}{diagonal_signal}'
                    t.sides[W].strength = 1
                    t.sides[S].label = f'{col}{col_mark}{diagonal_signal}'
                    t.sides[S].strength = 2
                    t.color = color_mapper(row, col, row_mark, col_mark)
                    tile_set[tile_type_name] = t

    return tile_set

    
if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hp:r:o:d:s",
                            ["help", "pattern=", "reps=", "out=", "dir=", "signals"])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()


    # Default values (used if not specified on the command line)
    pattern = '01101011'
    num_repetitions = 10

    base_file_name = None
    output_directory = '.'

    b_mark_signals = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-p', '--pattern']:
            pattern = p
        elif o in ['-r', '--reps']:
            num_repetiions = int(p)
        elif o in ['-o', '--out']:
            base_file_name = p
        elif o in ['-d', '--dir']:
            output_directory = p
        elif o in ['-s', '--signals']:
            b_mark_signals = True
        else:
            DisplayHelp()

    if base_file_name is None:
        base_file_name = f'pattern_{pattern}_x{num_repetitions}'
    
    if not os.path.exists(output_directory):
        print(f'Creating output directory {output_directory}')
        os.makedirs(output_directory)
   
    expanded_pattern = ''
    for i in range(num_repetitions):
        for j in range(len(pattern)):
            if j == 0:
                expanded_pattern += pattern[j] + 'B'
            else:
                expanded_pattern += pattern[j] + 'I'            


    tas = PyTASTileSystem()

    diagonal_signal = '*'
    signal_symbols = [diagonal_signal]

    init_row_tile_set, init_row_tile_list = GetInitialRowTiles(expanded_pattern, ColorMapper, 2, diagonal_signal)
    tas.tileTypes.update(init_row_tile_set)

    interior_tiles = GetInteriorTiles(ColorMapper, diagonal_signal)
    tas.tileTypes.update(interior_tiles)

    if b_mark_signals:
        for tile_type_name in tas.tileTypes:
            tile_type = tas.tileTypes[tile_type_name]
            for symbol in signal_symbols:
                if tile_type.label.endswith(symbol):
                    tile_type.color = color_red

    x_coord = 0
    for t in init_row_tile_list:
        tas.seedPatch.append((t.name, (x_coord, 0)))
        x_coord += 1

    tas.SaveToFile(f'{output_directory}/{base_file_name}')
    print(f"Output TAS to files '{output_directory}/{base_file_name}' with {len(tas.tileTypes)} tile types for pattern '{pattern}' repeated {num_repetitions} times. ({len(init_row_tile_set)} for the seed row.)")

    


