from typing import List
from PyTASTileSystem import *


#     Direction Helpers 
# ==============================

def string_to_dir(s: str) -> int:
    if s == 'N': return N
    if s == 'E': return E
    if s == 'S': return S
    if s == 'W': return W
    raise Exception(f'Invalid direction: {s}')

def get_rot_dirs(dir: int):
    '''
    Returns all of the directions rotated to face the input direction
    '''
    if dir == N: 
        return [N, E, S, W]
    if dir == W: 
        return [W, N, E, S]
    if dir == E: 
        return [E, S, W, N]
    if dir == S:
        return [S, W, N, E]
    raise Exception(f'Invalid direction: {dir}')

def direction_to_string(dir: int) -> str:
    if dir == N:
        return 'N'
    elif dir == E:
        return 'E'
    elif dir == S:
        return 'S'
    elif dir == W:
        return 'W'
    else:
        raise ValueError(f'Invalid direction: {dir}')

def rotate_dir_ccw(dir: int) -> int:
    return (dir + 1) % 4

def get_opposite_dir(dir: int) -> int:
    return (dir + 2) % 4

def rotate_dir_cc(dir: int) -> int:
    return (dir + 3) % 4


#             Builders
# ==============================

def build_row(row_len: int, label: str, dir: int):
    '''
    Builds a row of tiles with the given label and direction.
    '''
    (N_, E_, S_, W_) = get_rot_dirs(dir)
    def make_name(x: str | int): 
        x = str(x)
        return f'{label}{x}'

    tiles: List[PyTASTileType] = []
    for i in range(row_len):
        t = PyTASTileType()
        t.name = make_name(i);
        t.label = make_name(i);
        if not i == row_len - 1:
            t.sides[N_] = Glue(2, make_name(i))
        if not i == 0:
            t.sides[S_] = Glue(2, make_name(i-1))
        tiles.append(t)

    return tiles
